from askanna.config.server import config as config_server
from askanna.config.project import config as config_project


class Config():

    @property
    def server(self):
        return config_server

    @property
    def project(self):
        return config_project


config = Config()



import os
import yaml

import click

from askanna.cli.utils import ask_which_workspace
from askanna.core.apiclient import client


HELP = """
This command will allow you to create an AskAnna project in your current directory
"""

SHORT_HELP = "Create a project in the current directory"


class CreateProject:
    def __init__(self, name: str = None):
        self.client = client
        self.name = name

    def cli(self, workspace_suuid: str = None, description: str = None):
        if not self.name:
            click.echo(
                "Hi! It is time to create a new project in AskAnna. "
                "We start with some information about the project.\n"
            )
            self.name = click.prompt("Project name", type=str)

            if not description:
                description = click.prompt(
                    "Project description", type=str, default="", show_default=False
                )

        if not workspace_suuid:
            workspace = ask_which_workspace("In which workspace do you want to create the new project?")
            workspace_suuid = workspace.short_uuid

        url = f"{self.client.base_url}project/"
        r = self.client.post(
            url,
            data={
                "name": self.name,
                "workspace": workspace_suuid,
                "description": description,
            },
        )
        if r.status_code == 201:
            click.echo("\nYou have successfully created a new project in AskAnna!")
            return r.json()
        else:
            raise Exception("We could not create the project.")


@click.command(help=HELP, short_help=SHORT_HELP)
@click.argument("name", required=False)
@click.option(
    "--workspace",
    "-w",
    show_default=True,
    help="Workspace SUUID where you want to create the project",
)
@click.option("--description", "-d", help="Description of the project")
def cli(name, workspace, description):
    cwd = os.getcwd()
    askanna_project_file = os.path.join(cwd, "askanna.yml")
    if os.path.exists(askanna_project_file):
        click.echo(
            "This directory already has an 'askanna.yml' file. If you continue, we will "
            "create a new project in AskAnna."
        )
        click.echo(
            "But, we will not update your 'askanna.yml' file with the push-target of the "
            "new project. If you continue, you need to add the push-target yourself.\n"
        )
        click.confirm(
            "Do you want to continue without updating the 'askanna.yml'?", abort=True
        )
        click.echo("")

    project_creator = CreateProject(name=name)
    project_info = project_creator.cli(workspace_suuid=workspace, description=description)

    if not os.path.exists(askanna_project_file):
        with open(askanna_project_file, "w") as pf:
            pf.write(yaml.dump({"push-target": project_info["url"]}, indent=2))
    else:
        click.echo(
            "\nTo be able to push your code to AskAnna, you need to update your local `askanna.yml` file. "
            "In the `askanna.yml` set or update the push-target with:"
        )
        click.echo("push-target: {}".format(project_info["url"]))

    # finish
    click.echo("\nCheck your project in AskAnna at:")
    click.echo("{project_url}".format(project_url=project_info["url"]))
    click.echo(
        "\nAs a first step you can configure your first job for this project in the `askanna.yml` file. "
        "Ones you are done, you can easily push your code to AskAnna via:\n"
        "\n"
        "  askanna push\n"
        "\n"
        "Success with your project!"
    )



from click.testing import CliRunner
from faker import Faker
from faker.providers import date_time, internet, file, misc
import os
import shutil
import tempfile
import unittest

from askanna.cli.tool import cli_commands
from askanna.config import config


fake = Faker()
fake.add_provider(date_time)
fake.add_provider(internet)
fake.add_provider(misc)
fake.add_provider(file)


class TestCliLogin(unittest.TestCase):
    """
    askanna login

    We expect to be able to setup authentication using arguments or via a prompt
    """

    def setUp(self):
        self.environ_bck = dict(os.environ)
        self.tempdir = tempfile.mkdtemp(prefix='askanna-test-cli-login')

        self.remote_url = 'https://' + fake.hostname()
        self.email = fake.email()
        self.password = fake.password(length=12)
        self.token_value = fake.password(length=40)

        config.server.server_config_path = self.tempdir + '/.askanna.yml'
        config.server.remote = self.remote_url
        config.server.token = self.token_value

    def tearDown(self):
        os.environ.clear()
        os.environ.update(self.environ_bck)

        shutil.rmtree(self.tempdir, ignore_errors=True)

    def test_command_logout_base(self):
        result = CliRunner().invoke(cli_commands, "logout --help")

        assert not result.exception
        assert result.exit_code == 0
        assert "logout" in result.output
        assert "token" in result.output

    def test_command_logout(self):
        assert config.server.token == self.token_value

        result = CliRunner().invoke(cli_commands, "logout")

        assert not result.exception
        assert result.exit_code == 0
        assert "You have been logged out" in result.output
        assert config.server.token == ''



import requests

from askanna.core.exceptions import ConnectionError


class Session:
    def __init__(self, headers=None, *args, **kwargs):
        self.session = requests.Session()

        if headers:
            self.session.headers.update(headers)

    def _connection_error_message(self, url, error):
        connection_error_message_base = 'Something went wrong. Please check whether the URL is an AskAnna backend.'
        return f'{connection_error_message_base}\n    URL:    {url}\n    Error: {error}'

    def get(self, url, **kwargs):
        try:
            return self.session.get(url, **kwargs)
        except requests.exceptions.ConnectionError as e:
            raise ConnectionError(self._connection_error_message(url, e))

    def head(self, url, **kwargs):
        try:
            return self.session.head(url, **kwargs)
        except requests.exceptions.ConnectionError as e:
            raise ConnectionError(self._connection_error_message(url, e))

    def patch(self, url, **kwargs):
        try:
            return self.session.patch(url, **kwargs)
        except requests.exceptions.ConnectionError as e:
            raise ConnectionError(self._connection_error_message(url, e))

    def put(self, url, **kwargs):
        try:
            return self.session.put(url, **kwargs)
        except requests.exceptions.ConnectionError as e:
            raise ConnectionError(self._connection_error_message(url, e))

    def post(self, url, **kwargs):
        try:
            return self.session.post(url, **kwargs)
        except requests.exceptions.ConnectionError as e:
            raise ConnectionError(self._connection_error_message(url, e))

    def create(self, url, **kwargs):
        try:
            return self.post(url, **kwargs)
        except requests.exceptions.ConnectionError as e:
            raise ConnectionError(self._connection_error_message(url, e))

    def delete(self, url, **kwargs):
        try:
            return self.session.delete(url, **kwargs)
        except requests.exceptions.ConnectionError as e:
            raise ConnectionError(self._connection_error_message(url, e))



"""
Management of workspaces in AskAnna
This is the class which act as gateway to the API of AskAnna
"""
import sys

import click

from askanna.core import exceptions
from askanna.core.apiclient import client
from askanna.core.dataclasses import Workspace


class WorkspaceGateway:
    def __init__(self, *args, **kwargs):
        self.client = client
        self.base_url = self.client.base_url + "workspace/"

    def list(self) -> list:
        r = self.client.get(self.base_url)

        if r.status_code != 200:
            raise exceptions.GetError(
                "{} - Something went wrong while retrieving "
                "workspaces: {}".format(r.status_code, r.json())
            )

        return [Workspace(**workspace) for workspace in r.json()]

    def change(
        self, suuid: str, name: str = None, description: str = None
    ) -> Workspace:
        """
        Change the workspace information
        """
        url = f"{self.base_url}{suuid}/"

        changes = [
            ["name", name],
            ["description", description],
        ]
        # filterout None's
        changes = list(filter(lambda x: x[1] is not None, changes))
        if len(changes) == 0:
            click.echo(
                "Nothing to change for this workspace. You did not provide a name or description.",
                err=True,
            )
            sys.exit(1)

        r = self.client.patch(url, json=dict(changes))
        if r.status_code == 200:
            workspace = Workspace(**r.json())
            click.echo(f"You have successfully changed the workspace: {workspace.name}")
            return workspace
        else:
            raise exceptions.PatchError(
                "{} - Something went wrong while updating the workspace information: "
                "{}".format(r.status_code, r.json())
            )



import os


DEFAULT_SERVER_CONFIG_PATH = os.path.expanduser('~/.askanna.yml')
DEFAULT_SERVER_REMOTE = 'https://beta-api.askanna.eu'
DEFAULT_SERVER_UI = 'https://beta.askanna.eu'

DEFAULT_PROJECT_TEMPLATE = 'https://gitlab.com/askanna/project-templates/blanco-template.git'

PYPI_PROJECT_URL = 'https://pypi.org/pypi/askanna/json'



# -*- coding: utf-8 -*-

"""Top-level package for askanna"""

__author__ = "AskAnna Team"
__email__ = "devops@askanna.io"
__version__ = "0.14.1"

import re
import sys

# Determine whether we are in the CLI or using the SDK
# we only check for the `askanna` executable
USING_ASKANNA_CLI: bool = any([re.match(".+bin/askanna$", sys.argv[0])])

try:
    import click  # noqa
except Exception as e:
    # We are propably within an installation
    print(e)
else:
    try:
        from dotenv import find_dotenv, load_dotenv
    except ImportError:
        pass  # we only use dotenv for development
    else:
        load_dotenv(find_dotenv())

    from .init import *  # noqa



import click

from askanna.config import config

HELP = """
Remove the AskAnna API token that is saved in your global configuration file.
"""

SHORT_HELP = "Remove saved AskAnna API token"


@click.command(help=HELP, short_help=SHORT_HELP)
def cli():
    config.server.logout_and_remove_token()
    click.echo("You have been logged out!")



import click
from cookiecutter.main import cookiecutter as cookiecreator
from cookiecutter.exceptions import OutputDirExistsException
import os
from slugify import slugify

from askanna.cli.utils import ask_which_workspace
from askanna.core.apiclient import client
from askanna.core.push import push
from askanna.settings import DEFAULT_PROJECT_TEMPLATE


HELP = """
This command will allow you to create an AskAnna project in a new directory
"""

SHORT_HELP = "Create a project in a new directory"


class CreateProject:
    def __init__(self, name: str = None):
        self.client = client
        self.name = name
        self.slugified_name = None
        if self.name:
            self.slugified_name = slugify(self.name)

    def cli(self, workspace_suuid: str = None, description: str = None):
        if not self.name:
            click.echo(
                "Hi! It is time to create a new project in AskAnna. "
                "We start with some information about the project.\n"
            )
            self.name = click.prompt("Project name", type=str)
            self.slugified_name = slugify(self.name)

            if not description:
                description = click.prompt(
                    "Project description", type=str, default="", show_default=False
                )

        if not workspace_suuid:
            workspace = ask_which_workspace("In which workspace do you want to create the new project?")
            workspace_suuid = workspace.short_uuid

        url = f"{self.client.base_url}project/"
        r = self.client.post(
            url,
            data={
                "name": self.name,
                "workspace": workspace_suuid,
                "description": description,
            },
        )
        if r.status_code == 201:
            click.echo("\nYou have successfully created a new project in AskAnna!")
            return r.json()
        else:
            raise Exception("We could not create the project.")


@click.command(help=HELP, short_help=SHORT_HELP)
@click.argument("name", required=False)
@click.option(
    "--workspace",
    "-w",
    show_default=True,
    help="Workspace SUUID where you want to create the project",
)
@click.option("--description", "-d", help="Description of the project [optional]")
@click.option(
    "--template",
    "-t",
    "project_template",
    help="Location of a Cookiecutter project template",
)
@click.option(
    "--push/--no-push",
    "-p",
    "is_push",
    default=False,
    show_default=False,
    help="Push an initial version of the code [default: no-push]",
)
def cli(name, workspace, description, project_template, is_push):
    project_creator = CreateProject(name=name)
    project_info = project_creator.cli(workspace_suuid=workspace, description=description)
    project_dir = project_creator.slugified_name
    if not project_template:
        project_template = DEFAULT_PROJECT_TEMPLATE

    try:
        cookiecreator(
            project_template,
            no_input=True,
            overwrite_if_exists=False,
            extra_context={
                "project_name": project_info["name"],
                "project_directory": project_dir,
                "project_push_target": project_info["url"],
            },
        )
    except OutputDirExistsException:
        click.echo(
            f"You already have a project directory '{project_dir}'."
            + " If you open the new project in AskAnna, you find instructions about "
            + "how you can push your project to AskAnna."
        )
    else:
        click.echo(f"Open your new local project directory: 'cd {project_dir}'")

        if is_push:
            click.echo("")  # print an empty line

            # also push the new directory to AskAnna
            os.chdir(project_dir)
            push(force=True, description="Initial push")

    # finish
    click.echo(
        "\nWe have setup the new project. You can check your project in AskAnna at:"
    )
    click.echo(project_info["url"])
    click.echo("\nSuccess with your project!")



from dataclasses import dataclass
import re
from typing import Dict

from askanna.config.utils import read_config, scan_config_in_path


@dataclass
class PushTarget:
    url: str = ''
    http_scheme: str = ''
    host: str = ''
    project_suuid: str = ''
    workspace_suuid: str = ''


@dataclass
class ProjectConfig:
    config_dict: Dict
    project_config_path: str = ''
    push_target: PushTarget = PushTarget()
    project_suuid: str = ''
    workspace_suuid: str = ''


def extract_push_target(push_target: str) -> PushTarget:
    """
    Extract push target from the url configured
    Workspace is optional
    """
    if not push_target:
        raise ValueError("Cannot extract push-target if push-target is not set.")
    match_pattern = re.compile(
        r"(?P<http_scheme>https|http):\/\/(?P<askanna_host>[\w\.\-\:]+)\/(?P<workspace_suuid>[\w-]+){0,1}\/{0,1}project\/(?P<project_suuid>[\w-]+)\/{0,1}"  # noqa: E501
    )
    matches = match_pattern.match(push_target)

    if not matches:
        raise ValueError(f"The push target '{push_target}' is not valid. Please check the documentation to read "
                         "more about the push target: https://docs.askanna.io/code/#push-target")

    matches_dict = matches.groupdict()

    return PushTarget(
        url=push_target,
        http_scheme=matches_dict['http_scheme'],
        host=matches_dict['askanna_host'],
        workspace_suuid=matches_dict['workspace_suuid'],
        project_suuid=matches_dict['project_suuid'],
    )


def load_config(project_config_path: str = '') -> ProjectConfig:
    if not project_config_path:
        project_config_path = scan_config_in_path()

    project_config = {}
    push_target = PushTarget()
    workspace_suuid = ''
    project_suuid = ''

    if project_config_path:
        project_config = read_config(project_config_path)

    push_target_str = project_config.get('push-target')
    if push_target_str:
        push_target = extract_push_target(push_target_str)
        workspace_suuid = push_target.workspace_suuid
        project_suuid = push_target.project_suuid

    return ProjectConfig(
        config_dict=project_config,
        project_config_path=project_config_path,
        push_target=push_target,
        workspace_suuid=workspace_suuid,
        project_suuid=project_suuid,
    )


config: ProjectConfig = load_config()



from dataclasses import dataclass
import os
import sys
from typing import Dict


import click

from askanna.config.utils import read_config, store_config
from askanna.settings import (
    DEFAULT_SERVER_CONFIG_PATH,
    DEFAULT_SERVER_REMOTE,
    DEFAULT_SERVER_UI,
)


SERVER_CONFIG_PATH = os.getenv('AA_SERVER_CONFIG_FILE', DEFAULT_SERVER_CONFIG_PATH)
SERVER = os.getenv('AA_SERVER', 'default')
DEFAULT_SERVER_CONFIG = {
    'server': {
        'default': {
            'remote': f'{DEFAULT_SERVER_REMOTE}',
            'ui': f'{DEFAULT_SERVER_UI}',
            'token': '',
        },
    }
}


@dataclass
class ServerConfig:
    """
    Server configuration specification
    """

    remote: str
    config_dict: Dict = None
    server_config_path: str = SERVER_CONFIG_PATH
    server: str = 'default'
    token: str = ''
    ui: str = ''

    @property
    def is_authenticated(self) -> bool:
        if self.token:
            return True
        return False

    def save_server_to_config_file(self):
        self.config_dict = add_or_update_server_in_config_dict(
            self.config_dict,
            self.server,
            self.remote,
            self.token,
            self.ui
        )
        store_config(self.server_config_path, self.config_dict)

    def logout_and_remove_token(self):
        self.token = ''
        self.save_server_to_config_file()


def add_or_update_server_in_config_dict(
    config_dict: Dict,
    server: str,
    remote: str,
    token: str = '',
    ui: str = ''
) -> Dict:

    server_dict = {
        'remote': remote,
        'ui': ui,
        'token': token,
    }

    if config_dict.get('server', {}).get(server):
        config_dict['server'][server].update(server_dict)
    elif config_dict.get('server'):
        config_dict['server'].update({server: server_dict})
    else:
        config_dict['server'] = {server: server_dict}

    return config_dict


def update_to_new_config(config_dict : Dict, server_config_path : str, server : str):
    """
    Updates the config file because the AskAnna team changed the config file setup
    This function can be removed in the future.
    """

    remote = config_dict['askanna'].get('remote', '')
    remote = remote.replace('/v1/', '')

    ui = ''
    token = ''

    if remote == 'https://beta-api.askanna.eu':
        ui = 'https://beta.askanna.eu'

    del config_dict['askanna']

    if config_dict.get('auth'):
        token = config_dict['auth'].get('token', '')

    try:
        del config_dict['auth']
    except:  # noqa
        pass

    config_dict = add_or_update_server_in_config_dict(config_dict, server, remote, token, ui)

    store_config(server_config_path, config_dict)
    click.echo('[INFO] We updated your AskAnna config file to support the latest features')

    return read_config(server_config_path)


def load_config(server_config_path : str = SERVER_CONFIG_PATH, server : str = SERVER) -> ServerConfig:
    config_dict = DEFAULT_SERVER_CONFIG

    if os.path.isfile(server_config_path):
        local_config = read_config(server_config_path)
        config_dict.update(local_config)

        # Automatically updates the config file because of new config setup
        # Next section can be removed in a future version
        if config_dict.get('askanna'):
            config_dict = update_to_new_config(config_dict, server_config_path, server)

    server_dict = config_dict.get('server', {}).get(server, {})
    if not server_dict and server != 'default':
        click.echo(f"Server settings for '{server}' not found. Please try again with a valid server, or add this "
                   "server.", err=True)
        sys.exit(1)

    remote = os.getenv('AA_REMOTE', server_dict.get('remote', DEFAULT_SERVER_REMOTE))
    ui = os.getenv('AA_UI', server_dict.get('ui', DEFAULT_SERVER_UI))
    token = os.getenv('AA_TOKEN', server_dict.get('token', ''))

    return ServerConfig(
        config_dict=config_dict,
        server_config_path=server_config_path,
        server=server,
        remote=remote,
        ui=ui,
        token=token,
    )


config: ServerConfig = load_config()



"""
Management of variables in AskAnna
This is the class which act as gateway to the API of AskAnna
"""
import sys
import click

from askanna.core.apiclient import client
from askanna.core.dataclasses import Variable


class VariableGateway:
    def __init__(self, *args, **kwargs):
        self.client = client
        self.project = None
        self.base_url = self.client.base_url + "variable/"

    def list(self, project_suuid: str = None) -> list:
        if project_suuid:
            # build url to select for project only
            url = f"{self.client.base_url}project/{project_suuid}/variables/"
        else:
            url = self.base_url

        r = self.client.get(url)
        if r.status_code != 200:
            click.echo("We cannot find variables for you")
            sys.exit(1)

        variables = [Variable(**var) for var in r.json()]

        return variables

    def detail(self, suuid: str) -> Variable:
        url = f"{self.base_url}{suuid}/"
        r = self.client.get(url)
        return Variable(**r.json())

    def create(self, name: str, value: str, is_masked: bool, project_suuid: str):
        url = self.base_url
        r = self.client.create(url, json={
            "name": name,
            "value": value,
            "is_masked": is_masked,
            "project": project_suuid
        })
        if r.status_code == 201:
            return Variable(**r.json()), True

        return None, False

    def change(self, suuid: str, name: str = None, value: str = None,
               is_masked: bool = None) -> Variable:
        url = f"{self.base_url}{suuid}/"

        variable = {}
        if name:
            variable.update({"name": name})
        if value:
            variable.update({"value": value})
        if is_masked:
            variable.update({"is_masked": is_masked})

        if variable:
            # emit update request (PATCH)
            r = self.client.patch(
                url,
                json=variable
            )
        else:
            click.echo("Nothing to change for this variable. You did not provide a name, value or is_masked to change "
                       "for this variable", err=True)
            sys.exit(1)

        # show success or failure
        if r.status_code == 200:
            variable_name = r.json().get('name')
            click.echo(f"You have successfully changed the variable: '{variable_name}'")
        else:
            click.echo("Something went wrong while changing the variable", err=True)
            sys.exit(1)
        return Variable(**r.json())

    def delete(self, suuid: str) -> bool:
        url = f"{self.base_url}{suuid}/"
        r = self.client.delete(url)
        return r.status_code == 204



from askanna.config.utils import read_config_from_url
from askanna.core.apiclient import client
from askanna.core.dataclasses import User
from askanna.core.exceptions import GetError, PostError


class AuthGateway:
    """
    Authentication management for AskAnna CLI & SDK
    """

    def __init__(self):
        self.base_url = client.base_url.replace("/v1/", "")

    def login(
        self,
        email: str,
        password: str,
        remote_url: str = "",
        ui_url: str = "",
        server: str = "default",
        update_config_file: bool = False,
    ) -> None:
        client.config.server.server = server

        if ui_url:
            if ui_url[-1] == "/":
                ui_url = ui_url[:-1]
            ui_config = read_config_from_url(f"{ui_url}/askanna-config.yml")
            try:
                ui_remote_url = ui_config["askanna-remote"]
            except KeyError as e:
                raise KeyError(f"The config file on the URL '{ui_url}' did not contain the key {e}")

            if remote_url and remote_url != ui_remote_url:
                raise ValueError("The remote URL does not match the remote URL configured on the frontend")

            client.config.server.ui = ui_url
            remote_url = ui_remote_url

        if remote_url:
            self.base_url = remote_url.replace("/v1/", "")
            client.config.server.remote = self.base_url

        if self.base_url == "https://beta-api.askanna.eu" and not ui_url:
            client.config.server.ui = "https://beta.askanna.eu"

        url = f"{self.base_url}/rest-auth/login/"
        r = client.post(url, json={"username": email.strip(), "password": password.strip()})

        if r.status_code == 400:
            raise PostError("{} - We could not log you in. Please check your credentials.".format(r.status_code))
        if r.status_code == 404:
            raise PostError(
                "{} - We could not log you in. Please check the url or remote you provided.".format(r.status_code)
            )
        if r.status_code != 200:
            raise PostError("{} - We could not log you in: {}".format(r.status_code, r.reason))

        client.config.server.token = str(r.json().get("key"))
        client.update_session()

        if update_config_file:
            client.config.server.save_server_to_config_file()

    def get_user_info(self) -> User:
        url = f"{self.base_url}/rest-auth/user/"
        r = client.get(url)

        if r.status_code == 200:
            return User(**r.json())
        elif r.status_code == 401:
            raise GetError(
                "The provided token is not valid. Via `askanna logout` you can remove the token "
                "and via `askanna login` you can set a new token."
            )
        else:
            raise GetError("{} - We could not connect to AskAnna. More info:\n" "{}".format(r.status_code, r.reason))



import os
import shutil
import tempfile
import unittest

import responses
from click.testing import CliRunner
from faker import Faker
from faker.providers import date_time, file, internet, misc

from askanna.cli.tool import cli_commands
from askanna.config import config

fake = Faker()
fake.add_provider(date_time)
fake.add_provider(internet)
fake.add_provider(misc)
fake.add_provider(file)


class TestCliLogin(unittest.TestCase):
    """
    askanna login

    We expect to be able to setup authentication using arguments or via a prompt
    """

    def setUp(self):
        self.environ_bck = dict(os.environ)
        self.tempdir = tempfile.mkdtemp(prefix="askanna-test-cli-login")

        self.ui_url = "https://" + fake.hostname()
        self.ui_url_with_slash = self.ui_url + "/"
        self.wrong_config_url = "https://" + fake.hostname()
        self.remote_url = "https://" + fake.hostname()
        self.email = fake.email()
        self.password = fake.password(length=12)
        self.token_value = fake.password(length=40)

        config.server.server_config_path = self.tempdir + "/.askanna.yml"
        config.server.remote = self.remote_url
        config.server.token = ""

        self.responses = responses.RequestsMock()
        self.responses.start()
        self.responses.add(
            responses.POST,
            url=self.remote_url + "/rest-auth/login/",
            status=200,
            json={"key": self.token_value},
        )
        self.responses.add(
            responses.GET,
            url=self.remote_url + "/rest-auth/user/",
            status=200,
            json={
                "name": fake.name(),
                "email": self.email,
                "short_uuid": fake.uuid4(),
                "is_active": True,
                "date_joined": fake.iso8601(),
                "last_login": fake.iso8601(),
            },
        )
        self.responses.add(
            responses.GET,
            url=f"{self.ui_url}/askanna-config.yml",
            stream=True,
            content_type="application/octet-stream",
            status=200,
            body=f"askanna-remote: {self.remote_url}\n",
        )
        self.responses.add(
            responses.GET,
            url=f"{self.wrong_config_url}/askanna-config.yml",
            stream=True,
            content_type="application/octet-stream",
            status=200,
            body=f"no-remote: {self.remote_url}\n",
        )

    def tearDown(self):
        self.responses.stop
        self.responses.reset

        os.environ.clear()
        os.environ.update(self.environ_bck)

        shutil.rmtree(self.tempdir, ignore_errors=True)

    def test_command_login_base(self):
        result = CliRunner().invoke(cli_commands, "login --help")

        assert not result.exception
        assert result.exit_code == 0
        assert "login" in result.output
        assert "email" in result.output
        assert "password" in result.output
        assert "url" in result.output
        assert "remote" in result.output
        assert "noop" not in result.output

    def test_command_login_url(self):
        result = CliRunner().invoke(
            cli_commands, f"login --email {self.email} --password {self.password} --url {self.ui_url}"
        )

        assert not result.exception
        assert result.exit_code == 0
        assert self.email in result.output
        assert self.password not in result.output
        assert self.token_value not in result.output

    def test_command_login_url_with_slash(self):
        result = CliRunner().invoke(
            cli_commands, f"login --email {self.email} --password {self.password} --url {self.ui_url_with_slash}"
        )

        assert not result.exception
        assert result.exit_code == 0
        assert self.email in result.output
        assert self.password not in result.output
        assert self.token_value not in result.output

    def test_command_login_url_wrong_config(self):
        result = CliRunner().invoke(
            cli_commands, f"login --email {self.email} --password {self.password} --url {self.wrong_config_url}"
        )

        assert result.exception
        assert result.exit_code == 1
        assert self.password not in result.output
        assert self.token_value not in result.output

    def test_command_login_remote(self):
        result = CliRunner().invoke(
            cli_commands, f"login --email {self.email} --password {self.password} --remote {self.remote_url}"
        )

        assert not result.exception
        assert result.exit_code == 0
        assert self.email in result.output
        assert self.password not in result.output
        assert self.token_value not in result.output

    def test_command_login_url_and_remote(self):
        result = CliRunner().invoke(
            cli_commands,
            f"login --email {self.email} --password {self.password} --url {self.ui_url} --remote {self.remote_url}",
        )

        assert not result.exception
        assert result.exit_code == 0
        assert self.email in result.output
        assert self.password not in result.output
        assert self.token_value not in result.output

    def test_command_login_url_and_different_remote(self):
        different_remote = "https://" + fake.hostname()
        result = CliRunner().invoke(
            cli_commands,
            f"login --email {self.email} --password {self.password} --url {self.ui_url} --remote {different_remote}",
        )

        assert result.exception
        assert result.exit_code == 1
        assert self.password not in result.output
        assert self.token_value not in result.output

    def test_command_login_double(self):
        CliRunner().invoke(
            cli_commands, f"login --email {self.email} --password {self.password} --remote {self.remote_url}"
        )
        result = CliRunner().invoke(
            cli_commands, f"login --email {self.email} --password {self.password} --remote {self.remote_url}"
        )

        assert not result.exception
        assert result.exit_code == 0
        assert "already logged in" in result.output
        assert self.email in result.output
        assert "askanna logout" in result.output
        assert self.password not in result.output
        assert self.token_value not in result.output

    def test_command_ask_credentials(self):
        remote_url = "https://" + fake.hostname()
        email = fake.email()
        password = fake.password(length=12)
        token = fake.password(length=40)

        self.responses.add(
            responses.POST,
            url=remote_url + "/rest-auth/login/",
            status=200,
            json={"key": token},
        )
        self.responses.add(
            responses.GET,
            url=remote_url + "/rest-auth/user/",
            status=200,
            json={
                "name": "",
                "email": email,
                "short_uuid": fake.uuid4(),
                "is_active": True,
                "date_joined": fake.iso8601(),
                "last_login": fake.iso8601(),
            },
        )

        result = CliRunner().invoke(cli_commands, f"login --remote {remote_url}", input=f"{email}\n{password}")

        assert "Let's login into AskAnna" in result.output
        assert "Email:" in result.output
        assert "Password:" in result.output
        assert "logged in with" in result.output

    def test_command_ask_password(self):
        remote_url = "https://" + fake.hostname()
        email = fake.email()
        password = fake.password(length=12)
        token = fake.password(length=40)

        self.responses.add(
            responses.POST,
            url=remote_url + "/rest-auth/login/",
            status=200,
            json={"key": token},
        )
        self.responses.add(
            responses.GET,
            url=remote_url + "/rest-auth/user/",
            status=200,
            json={
                "name": "",
                "email": email,
                "short_uuid": fake.uuid4(),
                "is_active": True,
                "date_joined": fake.iso8601(),
                "last_login": fake.iso8601(),
            },
        )

        result = CliRunner().invoke(cli_commands, f"login --remote {remote_url} --email {email}", input=f"{password}")

        assert "Let's login into AskAnna" in result.output
        assert "Email:" not in result.output
        assert "Password:" in result.output
        assert "logged in with" in result.output

    def test_command_ask_email(self):
        remote_url = "https://" + fake.hostname()
        email = fake.email()
        password = fake.password(length=12)
        token = fake.password(length=40)

        self.responses.add(
            responses.POST,
            url=remote_url + "/rest-auth/login/",
            status=200,
            json={"key": token},
        )
        self.responses.add(
            responses.GET,
            url=remote_url + "/rest-auth/user/",
            status=200,
            json={
                "name": "",
                "email": email,
                "short_uuid": fake.uuid4(),
                "is_active": True,
                "date_joined": fake.iso8601(),
                "last_login": fake.iso8601(),
            },
        )

        result = CliRunner().invoke(
            cli_commands, f"login --remote {remote_url} --password {password}", input=f"{email}"
        )

        assert "Let's login into AskAnna" in result.output
        assert "Email:" in result.output
        assert "Password:" not in result.output
        assert "logged in with" in result.output
